@@ -155,9 +155,10 @@ def _set_filename(ctx, filename, msg=None):
 @click.command(cls=CommandsAliasedGroup, context_settings=_CONTEXT_SETTINGS)
 @click.option('-f', '--filename', type=str, default=None, show_default=True, help="Elf file path to pwn.")
 @click.option('-v', '--verbose', count=True, help="Show more info or not.")
-@click.version_option('1.2', "-V", "--version", prog_name='pwncli', message="%(prog)s: version %(version)s\nauthor: roderick chan\ngithub: https://github.com/RoderickChan/pwncli")
+@click.option('-E', '-ea', '--extra-argv', "extra_argv", type=str, default="", required=False, show_default=True, help="The extra argv for this script, sometimes it's useful for bruteforce.")
+@click.version_option('1.3', "-V", "--version", prog_name='pwncli', message="%(prog)s: version %(version)s\nauthor: roderick chan\ngithub: https://github.com/RoderickChan/pwncli")
 @pass_environ
-def cli(ctx, filename, verbose): # ctx: command property
+def cli(ctx, filename, verbose, extra_argv): # ctx: command property
     """pwncli tools for pwner!
 
     \b
@@ -203,6 +204,8 @@ def cli(ctx, filename, verbose): # ctx: command property
     ctx.gift['debug'] = False
     ctx.gift['remote'] = False
 
+    # extra argv for this script
+    ctx.gift['extra_argv'] = extra_argv
 
 def cli_script():
     cli.main(standalone_mode=False)
