@@ -12,14 +12,15 @@
 import threading
 import click
 from pwn import context, which, ELF, pause
+from pwnlib.atexit import register
 from pwnlib.gdb import attach
 import os
-import sys
+import re
 import string
 import tempfile
-import atexit
+from pwncli.utils.config import try_get_config_data_by_key
 from pwncli.cli import pass_environ, _set_filename
-from pwncli.utils.misc import ldd_get_libc_path
+from pwncli.utils.misc import ldd_get_libc_path, _in_tmux, _in_wsl, _get_gdb_plugin_info
 from pwncli.utils.cli_misc import CurrentGadgets
 
 _NO_TERMINAL = 0
@@ -28,35 +29,11 @@ _USE_OTHER_TERMINALS = 2
 _USE_GNOME_TERMINAL = 4
 
 
-def _in_tmux():
-    return bool('TMUX' in os.environ and which('tmux'))
-
-def _in_wsl():
-    if os.path.exists('/proc/sys/kernel/osrelease'):
-        with open('/proc/sys/kernel/osrelease', 'rb') as f:
-            is_in_wsl = b'icrosoft' in f.read()
-        if is_in_wsl and which('wsl.exe') and which("cmd.exe"):
-            return True
-    return False
-
 def __recover(f, c):
     # print("call recover")
     with open(f, "wb") as f2:
         f2.write(c)
 
-def _get_gdb_plugin_info():
-    with open(os.path.expanduser("~/.gdbinit"), "a+", encoding="utf-8") as f:
-        f.seek(0, 0)
-        for line in f:
-            if line.strip().startswith("source"):
-                if "pwndbg" in line:
-                    return "pwndbg"
-                elif "gef" in line:
-                    return "gef"
-                elif "peda" in line:
-                    return "peda"
-    return None
-
 
 def _set_gdb_type(pwncli_path, gdb_type):
     if gdb_type == 'auto':
@@ -127,7 +104,7 @@ def _set_terminal(ctx, p, flag, attach_mode, use_gdb, gdb_type, script, is_file,
         terminal = ['tmux', 'splitw', '-h']
     elif flag & _USE_GNOME_TERMINAL:
         terminal = ["gnome-terminal", "--", "sh", "-c"]
-    elif (flag & _USE_OTHER_TERMINALS) and which('cmd.exe'): # use cmd.exe to launch wt.exe bash.ex ...
+    elif (flag & _USE_OTHER_TERMINALS) and which('cmd.exe'): # use cmd.exe to launch wt.exe bash.exe ...
         if is_file:
             gdbcmd = " {}\"".format("-x " + gdb_script)
         else:
@@ -152,21 +129,15 @@ def _set_terminal(ctx, p, flag, attach_mode, use_gdb, gdb_type, script, is_file,
             os.system(cmd_use)
             return
         else:
-            ubu_name = ''
-            with open('/etc/issue', mode='rb') as f:
-                content = f.read()
-            if b'16.04' in content:
-                ubu_name = '16.04'
-            elif b'18.04' in content:
-                ubu_name = '18.04'
-            elif b'20.04' in content:
-                ubu_name = '20.04'
-            else:
-                ctx.abort('debug-command --> Only support ubuntu 16.04/18.04/20.04 in wsl')
-
-            distro_name = 'Ubuntu-{}'.format(ubu_name)
-            ubuntu_exe_name = 'ubuntu{}.exe'.format(ubu_name.replace('.', ""))
-            ctx.vlog2("debug-command --> Try to find wsl distro, name '{}'".format(distro_name))
+            distro_name = os.getenv('WSL_DISTRO_NAME')
+            if not distro_name:
+                ctx.abort('debug-command --> Cannot get distro name in wsl, please check your env!')
+            
+            if not re.search("ubuntu-\d\d.\d\d", distro_name, re.I):
+                ctx.abort('debug-command --> Only support Ubuntu-XX.XX system!')
+            
+            ctx.vlog2("debug-command --> Find wsl distro, name '{}'".format(distro_name))
+            ubuntu_exe_name = distro_name.lower().replace("-", "").replace(".", "") + ".exe"
             
             if attach_mode == 'wsl-u' and which(ubuntu_exe_name):
                 cmd_use = cmd.format(ubuntu_exe_name)
@@ -234,6 +205,20 @@ def _check_set_value(ctx, filename, argv, env, use_tmux, use_wsl, use_gnome, att
         argv = []
     
     # detect attach_mode
+    __attachmode_mapping = {
+        "t": "tmux",
+        "a": "auto",
+        "b": "wsl-b",
+        "u": "wsl-u",
+        "wt": "wsl-wt",
+        "wts": "wsl-wts",
+        "w": "wsl-w",
+        "o": "wsl-o",
+    }
+    for _k, _v in __attachmode_mapping.items():
+        if attach_mode == _k:
+            attach_mode = _v
+    
     if attach_mode.startswith('wsl'):
         use_wsl = True
 
@@ -242,16 +227,16 @@ def _check_set_value(ctx, filename, argv, env, use_tmux, use_wsl, use_gnome, att
     # check tmux
     if use_tmux:
         if not _in_tmux():
-            ctx.abort(msg="debug-command 'tmux' --> Not in tmux")
+            ctx.abort(msg="debug-command 'tmux' --> Not in tmux, please launch tmux first!")
         t_flag = _USE_TMUX
     # check wsl
     elif use_wsl:
         if not  _in_wsl():
-            ctx.abort(msg="debug-command 'wsl' --> Not in wsl")
+            ctx.abort(msg="debug-command 'wsl' --> Not in wsl, the option -w is only used for wsl!")
         t_flag = _USE_OTHER_TERMINALS
     elif use_gnome:
         if not which("gnome-terminal"):
-            ctx.abort(msg="debug-command 'gnome' --> No gnome-terminal")
+            ctx.abort(msg="debug-command 'gnome' --> No gnome-terminal, please install gnome-terminal first!")
         t_flag = _USE_GNOME_TERMINAL
 
     # process gdb-scripts
@@ -296,11 +281,15 @@ def _check_set_value(ctx, filename, argv, env, use_tmux, use_wsl, use_gnome, att
     # if gdb_script is file, then open it
     if is_file:
         if script:
-            with open("/tmp/pwncli_gdb_debug_file", 'w', encoding='utf-8') as f:
+            tmp_fd, tmp_gdb_script = tempfile.mkstemp(text=True)
+            ctx.vlog("debug-command --> Create a tempfile used for gdb_script, file path: {}".format(tmp_gdb_script))
+            os.close(tmp_fd)
+            register(lambda x: os.unlink(x), tmp_gdb_script)
+            with open(tmp_gdb_script, 'wt', encoding='utf-8') as f:
                 f.write(script +"\n")
-                with open(gdb_script, "r", encoding='utf-8') as f2:
+                with open(gdb_script, "rt", encoding='utf-8') as f2:
                     f.write(f2.read() + "\n")
-            gdb_script = "/tmp/pwncli_gdb_debug_file"
+            gdb_script = tmp_gdb_script
         script = open(gdb_script, "r", encoding="utf-8")
     
     if env:
@@ -339,7 +328,7 @@ int {}()
             if context.bits == 32:
                 cmd += " -m32"
             ctx.vlog("debug-command 'pause_before_main/hook_file' --> Execute cmd '{}'.".format(cmd))
-            atexit.register(lambda x: os.unlink(x) or os.unlink("{}.so".format(x)), tmp_path)
+            register(lambda x: os.unlink(x) or os.unlink("{}.so".format(x)), tmp_path)
             if not os.system(cmd):
                 ctx.vlog(msg="debug-command 'pause_before_main/hook_file' --> Execute last cmd success.")
                 if env:
@@ -408,7 +397,9 @@ int {}()
     if ctx.fromcli: 
         ctx.gift['io'].interactive()
     else:
-        threading.Thread(target=lambda :CurrentGadgets.reset(), daemon=True).start()
+        res = try_get_config_data_by_key(ctx.config_data, "debug", "load_gadget")
+        if res and res.strip().lower() in ("true", "yes", "enabled", "enable", "1"):
+            threading.Thread(target=lambda :CurrentGadgets.reset(), daemon=True).start()
 
 
 
@@ -422,7 +413,7 @@ int {}()
 @click.option('-t', '--use-tmux', '--tmux', "tmux", is_flag=True, show_default=True, help="Use tmux to gdb-debug or not.")
 @click.option('-w', '--use-wsl', '--wsl', "wsl", is_flag=True, show_default=True, help="Use wsl to pop up windows for gdb-debug or not.")
 @click.option('-g', '--use-gnome', '--gnome', "gnome", is_flag=True, show_default=True, help="Use gnome terminal to pop up windows for gdb-debug or not.")
-@click.option('-m', '-am', '--attach-mode', "attach_mode", type=click.Choice(['auto', 'tmux', 'wsl-b', 'wsl-u', 'wsl-o', 'wsl-wt', 'wsl-wts', 'wsl-w']), nargs=1, default='auto', show_default=True, help="Gdb attach mode, wsl: bash.exe | wsl: ubuntu1x04.exe | wsl: open-wsl.exe | wsl: wt.exe wsl.exe")
+@click.option('-m', '-am', '--attach-mode', "attach_mode", type=click.Choice(['auto', 'tmux', 'wsl-b', 'wsl-u', 'wsl-o', 'wsl-wt', 'wsl-wts', 'wsl-w', 'a', 't', 'w', 'wt', 'wts', 'b', 'o', 'u']), nargs=1, default='auto', show_default=True, help="Gdb attach mode, wsl: bash.exe | wsl: ubuntu1x04.exe | wsl: open-wsl.exe | wsl: wt.exe wsl.exe")
 @click.option('-u', '-ug', '--use-gdb', "use_gdb", is_flag=True, show_default=True, help="Use gdb possibly.")
 @click.option('-G', '-gt','--gdb-type', "gdb_type", type=click.Choice(['auto', 'pwndbg', 'gef', 'peda']), nargs=1, default='auto', help="Select a gdb plugin.")
 @click.option('-b', '-gb', '--gdb-breakpoint', "gdb_breakpoint", default=[], type=str, multiple=True, show_default=True, help="Set gdb breakpoints while gdb-debug is used, it should be a hex address or '\$rebase' addr or a function name. Multiple breakpoints are supported.")
