@@ -1,18 +1,22 @@
 
+import functools
 import os
-from threading import Lock
+from threading import Lock, Thread
 import time
 from pwncli.cli import gift
-from .misc import get_callframe_info, log2_ex, errlog_exit, log_code_base_addr, log_libc_base_addr, \
-    one_gadget_binary, get_segment_base_addr_by_proc_maps, recv_libc_addr, \
-    get_flag_when_get_shell
-from pwn import flat, asm
+from .misc import get_callframe_info, log_ex, log2_ex, errlog_exit, log_code_base_addr, log_libc_base_addr, \
+    one_gadget_binary, one_gadget, get_segment_base_addr_by_proc_maps, recv_libc_addr, \
+    get_flag_when_get_shell, ldd_get_libc_path
+from pwn import flat, asm, ELF, process, remote
 from .ropperbox import RopperBox, RopperArchType
+from .decorates import deprecated
 
 __all__ = [
     "stop",
     "S",
     "get_current_one_gadget",
+    "get_current_one_gadget_from_file",
+    "get_current_one_gadget_from_libc",
     "get_current_codebase_addr",
     "get_current_libcbase_addr",
     "get_current_stackbase_addr",
@@ -28,9 +32,11 @@ __all__ = [
     "set_current_libc_base_and_log",
     "set_current_code_base",
     "set_current_code_base_and_log",
-    "s", "sl", "sa", "sla", "ru", "rl","rs",
-    "rls", "rlc", "ra", "rr", "r", "rn", "ia", "ic",
-    "CurrentGadgets"
+    "set_remote_libc",
+    "copy_current_io",
+    "s", "sl", "sa", "sla", "st", "slt", "ru", "rl","rs",
+    "rls", "rlc", "rle", "ra", "rr", "r", "rn", "ia", "ic", "cr",
+    "CurrentGadgets", "load_currentgadgets_background"
     ]
 
 def stop(enable=True):
@@ -69,14 +75,29 @@ def stop(enable=True):
 S = stop
 
 #----------------------------useful function-------------------------
-def get_current_one_gadget(libc_base=0, more=False):
+def get_current_one_gadget_from_file(libc_base=0, more=False):
     """Get current filename's all one_gadget.
 
     """
     if not gift.get('filename', None):
         errlog_exit("Cannot get_current_one_gadget, filename is None!")
     res = [x + libc_base for x in one_gadget_binary(gift['filename'], more)]
-    log2_ex("Get one_gadget: {}".format([hex(x) for x in res]))
+    log_ex("Get one_gadget: {} from {}".format([hex(x) for x in res], ldd_get_libc_path(gift['filename'])))
+    return res
+
+@deprecated("please use 'get_current_one_gadget_from_file' and 'get_current_one_gadget_from_libc' instead.")
+def get_current_one_gadget(libc_base=0, more=False):
+    get_current_one_gadget_from_file(libc_base, more)
+    
+
+def get_current_one_gadget_from_libc(more=False):
+    """Get current all one_gadget from libc
+
+    """
+    if not gift.get('libc', None):
+        errlog_exit("Cannot get_current_one_gadget_from_libc, libc is None!")
+    res = [x + gift['libc'].address for x in one_gadget(gift['libc'].path, more)]
+    log_ex("Get one_gadget: {} from {}".format([hex(x) for x in res], gift['libc'].path))
     return res
 
 _cache_segment_base_addr = None
@@ -169,6 +190,7 @@ def recv_current_libc_addr(offset:int=0):
     return recv_libc_addr(gift['io'], bits=gift['elf'].bits, offset=offset)
 
 
+@deprecated
 def get_current_flag_when_get_shell(use_cat=True, start_str="flag{"):
     if not gift.get('io', None):
         errlog_exit("Can not get current libc addr because of no io.")
@@ -243,36 +265,74 @@ def set_current_code_base_and_log(addr: int, offset: int or str = 0):
     log_code_base_addr(res)
     return res
 
+
+def set_remote_libc(libc_so_path: str) -> ELF:
+    if not gift.get('remote'):
+        return
+    if not gift.get('io', None):
+        errlog_exit("Can not set remote libc because of no io.")
+    if os.path.exists(libc_so_path) and os.path.isfile(libc_so_path):
+        gift['libc'] = ELF(libc_so_path, checksec=False)
+        gift['libc'].address = 0
+        return gift['libc']
+    else:
+        errlog_exit("libc_so_path not exists!")
+
+
+def copy_current_io():
+    """Only used for debug command"""
+    io = None
+    if gift.get('debug'):
+        io = process(gift.filename)
+    elif gift.get('remote'):
+        io = remote(gift.io, gift.port)
+    else:
+        raise RuntimeError()
+    return io
+
 #-----------------------------io------------------------
-def s(data):
+def s(*args, **kwargs):
     """send"""
     io = gift.get("io", None)
     if io:
-        io.send(data)
+        io.send(*args, **kwargs)
 
-def sl(data):
+def sl(*args, **kwargs):
     """sendline"""
     io = gift.get("io", None)
     if io:
-        io.sendline(data)
+        io.sendline(*args, **kwargs)
 
-def sa(delim, data):
+def sa(*args, **kwargs):
     """sendafter"""
     io = gift.get("io", None)
     if io:
-        io.sendafter(delim, data)
+        io.sendafter(*args, **kwargs)
 
-def sla(delim, data):
+def sla(*args, **kwargs):
     """sendlineafter"""
     io = gift.get("io", None)
     if io:
-        io.sendlineafter(delim, data)
+        io.sendlineafter(*args, **kwargs)
 
-def ru(delim) -> bytes:
+def st(*args, **kwargs):
+    """sendthen"""
+    io = gift.get("io", None)
+    if io:
+        io.sendthen(*args, **kwargs)
+
+def slt(*args, **kwargs):
+    """sendlinethen"""
+    io = gift.get("io", None)
+    if io:
+        io.sendlinethen(*args, **kwargs)
+
+
+def ru(*args, **kwargs) -> bytes:
     """recvuntil"""
     io = gift.get("io", None)
     if io:
-        return io.recvuntil(delim)
+        return io.recvuntil(*args, **kwargs)
 
 def rl() -> bytes:
     """recvline"""
@@ -280,23 +340,29 @@ def rl() -> bytes:
     if io:
         return io.recvline()
 
-def rs(n) -> list:
+def rs(*args, **kwargs) -> list:
     """recvlines"""
     io = gift.get("io", None)
     if io:
-        return io.recvlines(n)
+        return io.recvlines(*args, **kwargs)
 
-def rls(delims) -> bytes:
+def rls(*args, **kwargs) -> bytes:
     """recvline_startswith"""
     io = gift.get("io", None)
     if io:
-        return io.recvline_startswith(delims)
+        return io.recvline_startswith(*args, **kwargs)
 
-def rlc(delims) -> bytes:
+def rle(*args, **kwargs) -> bytes:
+    """recvline_endswith"""
+    io = gift.get("io", None)
+    if io:
+        return io.recvline_endswith(*args, **kwargs)
+
+def rlc(*args, **kwargs) -> bytes:
     """recvline_contains"""
     io = gift.get("io", None)
     if io:
-        return io.recvline_contains(delims)
+        return io.recvline_contains(*args, **kwargs)
 
 def ra(timeout=5) -> bytes:
     """recvall"""
@@ -304,23 +370,23 @@ def ra(timeout=5) -> bytes:
     if io:
         return io.recvall(timeout)
 
-def rr(regex) -> bytes:
+def rr(*args, **kwargs) -> bytes:
     """recvregex"""
     io = gift.get("io", None)
     if io:
-        return io.recvregex(regex)
+        return io.recvregex(*args, **kwargs)
 
-def r() -> bytes:
+def r(*args, **kwargs) -> bytes:
     """recv"""
     io = gift.get("io", None)
     if io:
-        return io.recv()
+        return io.recv(*args, **kwargs)
 
-def rn(n) -> bytes:
+def rn(*args, **kwargs) -> bytes:
     """recvn"""
     io = gift.get("io", None)
     if io:
-        return io.recvn(n)
+        return io.recvn(*args, **kwargs)
 
 def ia():
     """interactive"""
@@ -334,6 +400,12 @@ def ic():
     if io:
         io.close()
 
+def cr() -> bool:
+    """can_recv"""
+    io = gift.get("io", None)
+    if io:
+        return io.can_recv()
+
 # ----------------------------------gadget----------------
 
 class CurrentGadgets:
@@ -348,9 +420,11 @@ class CurrentGadgets:
     _mutex = Lock()
 
     @staticmethod
-    def set_find_area(find_in_elf=True, find_in_libc=False):
+    def set_find_area(find_in_elf=True, find_in_libc=False, do_initial=False):
         CurrentGadgets.__find_in_elf = find_in_elf
         CurrentGadgets.__find_in_libc = find_in_libc
+        if do_initial:
+            CurrentGadgets._initial_ropperbox()
 
     @staticmethod
     def _initial_ropperbox() -> bool:
@@ -419,22 +493,33 @@ class CurrentGadgets:
         if not CurrentGadgets._initial_ropperbox(): 
             return 0
         func = getattr(CurrentGadgets.__internal_libcbox, func_name)
-        try:
-            res = func('elf')
-            if CurrentGadgets.__elf.pie:
-                res += CurrentGadgets.__elf.address
-            return res
-        except:
+        if CurrentGadgets.__find_in_elf:
+            try:
+                res = func('elf')
+                if CurrentGadgets.__elf.pie:
+                    res += CurrentGadgets.__elf.address
+                return res
+            except:
+                pass
+        
+        if CurrentGadgets.__find_in_libc:
             res = func('libc')
             if CurrentGadgets.__libc.pie:
                 res += CurrentGadgets.__libc.address
             return res
+        
+        if not CurrentGadgets.__find_in_elf and not CurrentGadgets.__find_in_libc:
+            errlog_exit("Have closed both elf finder and libc finder.")
+        errlog_exit("Cannot find gadget using '{}'.".format(func_name))
+
 
     @staticmethod
-    def find_gadget(find : str, find_type='asm', get_list=False) -> int:
+    @functools.lru_cache(maxsize=128, typed=True)
+    def find_gadget(find_str : str, find_type='asm', get_list=False) -> int:
         """ type: asm / opcode / string """
         if not CurrentGadgets._initial_ropperbox(): 
             return 0
+        find = find_str
         if find_type == "asm":
             find = asm(find).hex()
             func = getattr(CurrentGadgets.__internal_libcbox, "search_opcode")
@@ -444,20 +529,35 @@ class CurrentGadgets:
             func = getattr(CurrentGadgets.__internal_libcbox, "search_string")
         else:
             errlog_exit("Unsupported find_type, only: asm / opcode / string.")
-        try:
-            res = func(find ,'elf', get_list)
-            if CurrentGadgets.__elf.pie:
+        
+        res = None
+        if CurrentGadgets.__find_in_elf:
+            try:
+                res = func(find ,'elf', get_list)
+                _base = 0
+                if CurrentGadgets.__elf.pie:
+                    _base = CurrentGadgets.__elf.address
                 if get_list:
-                    return [i + CurrentGadgets.__elf.address for i in res]
+                    return [i + _base for i in res]
                 else:
-                    return CurrentGadgets.__elf.address + res
-        except:
+                    return _base + res
+            except:
+                pass
+
+        if CurrentGadgets.__find_in_libc:
             res = func(find ,'libc', get_list)
+            _base = 0
             if CurrentGadgets.__libc.pie:
-                if get_list:
-                    return [i + CurrentGadgets.__libc.address for i in res]
-                else:
-                    return CurrentGadgets.__libc.address + res 
+                _base = CurrentGadgets.__libc.address
+            if get_list:
+                return [i + _base for i in res]
+            else:
+                return _base + res
+
+        if not CurrentGadgets.__find_in_elf and not CurrentGadgets.__find_in_libc:
+            errlog_exit("Have closed both elf finder and libc finder.")
+        errlog_exit("Cannot find gadget: {}.".format(find_str))
+
 
     @staticmethod
     def syscall() -> int:
@@ -495,6 +595,11 @@ class CurrentGadgets:
         """pop rdx; ret"""
         return CurrentGadgets._internal_find('get_pop_rdx_ret')
 
+    @staticmethod
+    def pop_rdx_rbx_ret() -> int:
+        """pop rdx; pop rbx; ret"""
+        return CurrentGadgets._internal_find('get_pop_rdx_rbx_ret')
+
     @staticmethod
     def pop_rax_ret() -> int:
         """pop rax; ret"""
@@ -545,18 +650,27 @@ class CurrentGadgets:
     def sh() -> int:
         """sh"""
         return CurrentGadgets._internal_find('get_sh')
+    
+    @staticmethod
+    def __try_get_rdx_gadget(rdx_val, rbx_val=0) -> list:
+        try:
+            addr = CurrentGadgets.pop_rdx_ret()
+            return [addr, rdx_val]
+        except:
+            return [CurrentGadgets.pop_rdx_rbx_ret(), rdx_val, rbx_val]
 
     @staticmethod
     def execve_chain(bin_sh_addr=None) -> bytes:
-        CurrentGadgets._initial_ropperbox()
+        if not CurrentGadgets._initial_ropperbox():
+            return None
+        
         if CurrentGadgets.__arch == "i386":
             layout = [
                 CurrentGadgets.pop_rbx_ret(),
                 bin_sh_addr or CurrentGadgets.bin_sh(),
                 CurrentGadgets.pop_rcx_ret(),
                 0,
-                CurrentGadgets.pop_rdx_ret(),
-                0,
+                CurrentGadgets.__try_get_rdx_gadget(0, bin_sh_addr or CurrentGadgets.bin_sh()),
                 CurrentGadgets.pop_rax_ret(),
                 0xb,
                 CurrentGadgets.syscall()
@@ -567,8 +681,7 @@ class CurrentGadgets:
                 bin_sh_addr or CurrentGadgets.bin_sh(),
                 CurrentGadgets.pop_rsi_ret(),
                 0,
-                CurrentGadgets.pop_rdx_ret(),
-                0,
+                CurrentGadgets.__try_get_rdx_gadget(0),
                 CurrentGadgets.pop_rax_ret(),
                 0x3b,
                 CurrentGadgets.syscall()
@@ -580,15 +693,16 @@ class CurrentGadgets:
 
     @staticmethod
     def mprotect_chain(va, length=0x1000, prog=7) -> bytes:
-        CurrentGadgets._initial_ropperbox()
+        if not CurrentGadgets._initial_ropperbox():
+            return None
+        
         if CurrentGadgets.__arch == "i386":
             layout = [
                 CurrentGadgets.pop_rbx_ret(),
                 va,
                 CurrentGadgets.pop_rcx_ret(),
                 length,
-                CurrentGadgets.pop_rdx_ret(),
-                prog,
+                CurrentGadgets.__try_get_rdx_gadget(prog, va),
                 CurrentGadgets.pop_rax_ret(),
                 125,
                 CurrentGadgets.syscall()
@@ -599,8 +713,7 @@ class CurrentGadgets:
                 va,
                 CurrentGadgets.pop_rsi_ret(),
                 length,
-                CurrentGadgets.pop_rdx_ret(),
-                prog,
+                CurrentGadgets.__try_get_rdx_gadget(prog),
                 CurrentGadgets.pop_rax_ret(),
                 10,
                 CurrentGadgets.syscall()
@@ -608,4 +721,89 @@ class CurrentGadgets:
         else:
             errlog_exit("Unsupported arch: {}".format(CurrentGadgets.__arch))
         
-        return flat(layout)
\ No newline at end of file
+        return flat(layout)
+
+    @staticmethod
+    def orw_chain(flag_addr, buf_addr=None, flag_fd=3, write_fd=1, buf_len=0x30) -> bytes:
+        if not CurrentGadgets._initial_ropperbox():
+            return None
+        
+        if not buf_addr:
+            buf_addr = flag_addr
+        
+        if CurrentGadgets.__arch == "i386":
+            layout = [
+                # open
+                CurrentGadgets.pop_rbx_ret(),
+                flag_addr,
+                CurrentGadgets.pop_rcx_ret(),
+                0,
+                CurrentGadgets.pop_rax_ret(),
+                5,
+                CurrentGadgets.syscall_ret(),
+                # read
+                CurrentGadgets.pop_rbx_ret(),
+                flag_fd,
+                CurrentGadgets.pop_rcx_ret(),
+                buf_addr,
+                CurrentGadgets.__try_get_rdx_gadget(buf_len, flag_fd),
+                CurrentGadgets.pop_rax_ret(),
+                3,
+                CurrentGadgets.syscall_ret(),
+                # write
+                CurrentGadgets.pop_rbx_ret(),
+                write_fd,
+                CurrentGadgets.pop_rax_ret(),
+                4,
+                CurrentGadgets.syscall_ret(),
+            ]
+        elif CurrentGadgets.__arch == "amd64":
+            layout = [
+                # open
+                CurrentGadgets.pop_rdi_ret(),
+                flag_addr,
+                CurrentGadgets.pop_rsi_ret(),
+                0,
+                CurrentGadgets.pop_rax_ret(),
+                2,
+                CurrentGadgets.syscall_ret(),
+                # read
+                CurrentGadgets.pop_rdi_ret(),
+                flag_fd,
+                CurrentGadgets.pop_rsi_ret(),
+                buf_addr,
+                CurrentGadgets.__try_get_rdx_gadget(buf_len),
+                CurrentGadgets.pop_rax_ret(),
+                0x0,
+                CurrentGadgets.syscall_ret(),
+                # write
+                CurrentGadgets.pop_rdi_ret(),
+                write_fd,
+                CurrentGadgets.pop_rax_ret(),
+                0x1,
+                CurrentGadgets.syscall_ret(),
+            ]
+        else:
+            errlog_exit("Unsupported arch: {}".format(CurrentGadgets.__arch))
+        
+        return flat(layout)
+
+
+    @staticmethod
+    def write_by_magic(write_addr: int, ori: int, expected: int) -> bytes:
+        if not CurrentGadgets._initial_ropperbox():
+            return None
+        if CurrentGadgets.__arch == "amd64":
+            return flat([
+                CurrentGadgets.find_gadget("pop rbx; pop rbp; pop r12; pop r13;"),
+                expected - ori if expected > ori else expected - ori + 0x100000000,
+                write_addr+0x3d, 0, 0, 0, 0,
+                CurrentGadgets.magic_gadget()
+            ])
+            
+        else:
+            errlog_exit("Only used for amd64!")
+
+
+def load_currentgadgets_background(find_in_elf=True, find_in_libc=True):
+    Thread(target=CurrentGadgets.set_find_area, args=(find_in_elf, find_in_libc, True),daemon=True).start()
\ No newline at end of file
