@@ -15,4 +15,10 @@ class PwncliExit(SystemExit):
     """
     PwncliExit
     """
+    pass
+
+class PwncliTodoException(Exception):
+    """
+    TODO Exception
+    """
     pass
\ No newline at end of file
